"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.computeYogaTree = void 0;
const computeYogaNode_1 = require("./computeYogaNode");
const zIndex_1 = require("../utils/zIndex");
const walkTree = (bridge) => (tree, context) => {
    const { node, stop } = computeYogaNode_1.computeYogaNode(bridge)(tree, context);
    if (typeof tree === 'string' || tree.type === 'sketch_svg') {
        // handle svg node, eg: stop here, we will handle the children in the renderer
        return node;
    }
    if (tree.children && tree.children.length > 0) {
        // Calculates zIndex order
        const children = zIndex_1.zIndex(tree.children);
        for (let index = 0; index < children.length; index += 1) {
            const childComponent = children[index];
            // Avoid going into <text> node's children
            if (!stop) {
                const childNode = walkTree(bridge)(childComponent, context.forChildren());
                node.insertChild(childNode, index);
            }
        }
    }
    return node;
};
exports.computeYogaTree = (bridge) => (root, context) => walkTree(bridge)(root, context);
