"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.makeSvgLayer = void 0;
const sketch_file_format_ts_1 = require("@sketch-hq/sketch-file-format-ts");
const svg_model_1 = __importDefault(require("@lona/svg-model"));
const shapeLayers_1 = require("../shapeLayers");
const models_1 = require("../models");
const borders_1 = require("../borders");
const layerGroup_1 = require("../layerGroup");
const path_1 = require("./graphics/path");
const rect_1 = require("./graphics/rect");
function makeLayerFromPathElement(pathElement, _parentFrame, scale) {
    const { data: { params: { commands, style }, }, } = pathElement;
    // Paths are created using the original frame
    const pathFrame = rect_1.makeBoundingRectFromCommands(commands);
    const paths = path_1.makePathsFromCommands(commands, pathFrame);
    // Scale the frame to fill the viewBox
    const shapeGroupFrame = rect_1.scaleRect(pathFrame, scale);
    // Each shape path has an origin of {0, 0}, since the shapeGroup layer stores the real origin,
    // and we don't want to apply the origin translation twice.
    const shapePathFrame = models_1.makeRect(0, 0, shapeGroupFrame.width, shapeGroupFrame.height);
    const shapePaths = paths.map((path) => shapeLayers_1.makeShapePath(shapePathFrame, path));
    const viewStyle = {};
    if (style.fill) {
        viewStyle.backgroundColor = style.fill;
    }
    const shapeGroup = shapeLayers_1.makeShapeGroup(shapeGroupFrame, shapePaths, viewStyle);
    if (style.stroke && shapeGroup.style) {
        const lineCap = path_1.makeLineCapStyle(style.strokeLineCap);
        const borderStyle = borders_1.createUniformBorder(style.strokeWidth * scale, style.stroke, undefined, sketch_file_format_ts_1.FileFormat1.BorderPosition.Center, lineCap, lineCap);
        shapeGroup.style = Object.assign(Object.assign({}, shapeGroup.style), borderStyle);
    }
    return shapeGroup;
}
function makeLayerGroup(frame, layers, name) {
    const group = layerGroup_1.layerGroup(frame.x, frame.y, frame.width, frame.height, 1);
    group.name = name;
    group.layers = layers;
    return group;
}
function makeSvgLayer(layout, name, svg) {
    const { data: { params, children }, } = svg_model_1.default(svg);
    const { viewBox = {
        x: layout.left,
        y: layout.top,
        width: layout.width,
        height: layout.height,
    }, } = params;
    // Determine the rect to generate layers within
    const croppedRect = rect_1.resize(viewBox, layout, 'contain');
    const scale = croppedRect.width / viewBox.width;
    // The top-level frame is the union of every path within
    const frame = rect_1.unionRects(...children.map((pathElement) => rect_1.makeBoundingRectFromCommands(pathElement.data.params.commands)));
    // Scale the frame to fill the viewBox
    const scaledFrame = rect_1.scaleRect(frame, scale);
    const layers = children.map((element) => makeLayerFromPathElement(element, scaledFrame, scale));
    return makeLayerGroup(croppedRect, layers, name);
}
exports.makeSvgLayer = makeSvgLayer;
