"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.makeTextLayer = exports.parseTextStyle = exports.makeTextStyle = exports.FONT_STYLES = exports.TEXT_TRANSFORM = exports.TEXT_DECORATION_LINETHROUGH = exports.TEXT_ALIGN = exports.TEXT_DECORATION_UNDERLINE = void 0;
const sketch_file_format_ts_1 = require("@sketch-hq/sketch-file-format-ts");
const resizeConstraint_1 = require("./resizeConstraint");
const models_1 = require("./models");
const style_1 = require("./style");
exports.TEXT_DECORATION_UNDERLINE = {
    none: sketch_file_format_ts_1.FileFormat1.UnderlineStyle.None,
    underline: sketch_file_format_ts_1.FileFormat1.UnderlineStyle.Underlined,
    double: 9,
    'line-through': 0,
};
exports.TEXT_ALIGN = {
    auto: sketch_file_format_ts_1.FileFormat1.TextHorizontalAlignment.Left,
    left: sketch_file_format_ts_1.FileFormat1.TextHorizontalAlignment.Left,
    right: sketch_file_format_ts_1.FileFormat1.TextHorizontalAlignment.Right,
    center: sketch_file_format_ts_1.FileFormat1.TextHorizontalAlignment.Centered,
    justify: sketch_file_format_ts_1.FileFormat1.TextHorizontalAlignment.Justified,
};
const TEXT_ALIGN_REVERSE = {
    [sketch_file_format_ts_1.FileFormat1.TextHorizontalAlignment.Natural]: 'left',
    [sketch_file_format_ts_1.FileFormat1.TextHorizontalAlignment.Right]: 'right',
    [sketch_file_format_ts_1.FileFormat1.TextHorizontalAlignment.Centered]: 'center',
    [sketch_file_format_ts_1.FileFormat1.TextHorizontalAlignment.Justified]: 'justify',
};
exports.TEXT_DECORATION_LINETHROUGH = {
    none: 0,
    underline: 0,
    double: 0,
    'line-through': 1,
};
exports.TEXT_TRANSFORM = {
    uppercase: sketch_file_format_ts_1.FileFormat1.TextTransform.Uppercase,
    lowercase: sketch_file_format_ts_1.FileFormat1.TextTransform.Lowercase,
    initial: sketch_file_format_ts_1.FileFormat1.TextTransform.None,
    inherit: sketch_file_format_ts_1.FileFormat1.TextTransform.None,
    none: sketch_file_format_ts_1.FileFormat1.TextTransform.None,
    capitalize: sketch_file_format_ts_1.FileFormat1.TextTransform.None,
};
// this borrows heavily from react-native's RCTFont class
// thanks y'all
// https://github.com/facebook/react-native/blob/master/React/Views/RCTFont.mm
exports.FONT_STYLES = {
    normal: false,
    italic: true,
    oblique: true,
};
const makeFontDescriptor = (bridge) => (style) => ({
    _class: 'fontDescriptor',
    attributes: {
        name: bridge.findFontName(style),
        size: style.fontSize || 14,
    },
});
const makeTextStyleAttributes = (bridge) => (style) => (Object.assign(Object.assign(Object.assign({ underlineStyle: style.textDecoration ? exports.TEXT_DECORATION_UNDERLINE[style.textDecoration] || 0 : 0, strikethroughStyle: style.textDecoration
        ? exports.TEXT_DECORATION_LINETHROUGH[style.textDecoration] || 0
        : 0, paragraphStyle: Object.assign({ _class: 'paragraphStyle', alignment: exports.TEXT_ALIGN[style.textAlign || 'auto'], paragraphSpacing: style.paragraphSpacing || 0 }, (typeof style.lineHeight !== 'undefined' && style.lineHeight !== null
        ? {
            minimumLineHeight: style.lineHeight,
            maximumLineHeight: style.lineHeight,
            lineHeightMultiple: 1.0,
        }
        : {})) }, (typeof style.letterSpacing !== 'undefined' && style.letterSpacing !== null
    ? {
        kerning: style.letterSpacing,
    }
    : {})), (typeof style.textTransform !== 'undefined' && style.textTransform !== null
    ? {
        MSAttributedStringTextTransformAttribute: exports.TEXT_TRANSFORM[style.textTransform] * 1,
    }
    : {})), { MSAttributedStringFontAttribute: makeFontDescriptor(bridge)(style), textStyleVerticalAlignmentKey: 0, MSAttributedStringColorAttribute: models_1.makeColorFromCSS(style.color || 'black') }));
const makeAttribute = (bridge) => (node, location) => ({
    _class: 'stringAttribute',
    location,
    length: node.content.length,
    attributes: makeTextStyleAttributes(bridge)(node.textStyles),
});
const makeAttributedString = (bridge) => (textNodes) => {
    const json = {
        _class: 'attributedString',
        string: '',
        attributes: [],
    };
    let location = 0;
    textNodes.forEach((node) => {
        json.attributes.push(makeAttribute(bridge)(node, location));
        json.string += node.content;
        location += node.content.length;
    });
    return json;
};
exports.makeTextStyle = (bridge) => (style, shadows) => {
    const json = style_1.makeStyle(style, undefined, shadows);
    json.textStyle = {
        _class: 'textStyle',
        encodedAttributes: makeTextStyleAttributes(bridge)(style),
        verticalAlignment: sketch_file_format_ts_1.FileFormat1.TextVerticalAlignment.Top,
    };
    return json;
};
exports.parseTextStyle = (json) => {
    const style = style_1.parseStyle(json);
    if (json.textStyle) {
        const attr = json.textStyle.encodedAttributes;
        if (attr.underlineStyle) {
            style.textDecoration = attr.underlineStyle === 9 ? 'double' : 'underline';
        }
        if (attr.strikethroughStyle) {
            style.textDecoration = 'line-through';
        }
        if (attr.paragraphStyle &&
            attr.paragraphStyle.alignment &&
            TEXT_ALIGN_REVERSE[attr.paragraphStyle.alignment]) {
            style.textAlign = TEXT_ALIGN_REVERSE[attr.paragraphStyle.alignment];
        }
        if (attr.paragraphStyle && typeof attr.paragraphStyle.minimumLineHeight !== 'undefined') {
            style.lineHeight = attr.paragraphStyle.minimumLineHeight;
        }
        if (typeof attr.kerning !== 'undefined') {
            style.letterSpacing = attr.kerning;
        }
        const color = json.textStyle.encodedAttributes.MSAttributedStringColorAttribute;
        if (color) {
            style.color = `#${Math.round(color.red * 255).toString(16)}${Math.round(color.green * 255).toString(16)}${Math.round(color.blue * 255).toString(16)}`;
            if (color.alpha !== 1) {
                style.color += `${Math.round(color.alpha * 255).toString(16)}`;
            }
        }
        if (json.textStyle.encodedAttributes.MSAttributedStringTextTransformAttribute !==
            sketch_file_format_ts_1.FileFormat1.TextTransform.None) {
            style.textTransform =
                json.textStyle.encodedAttributes.MSAttributedStringTextTransformAttribute ===
                    sketch_file_format_ts_1.FileFormat1.TextTransform.Lowercase
                    ? 'lowercase'
                    : 'uppercase';
        }
        const font = json.textStyle.encodedAttributes.MSAttributedStringFontAttribute;
        style.fontSize = font.attributes.size;
        // we are cheating here, setting the name of the font instead of parsing
        // the family, weight and traits. react-sketchapp will handle it nevertheless
        style.fontFamily = font.attributes.name;
    }
    return style;
};
exports.makeTextLayer = (bridge) => (frame, name, textNodes, _style, resizingConstraint, shadows) => ({
    _class: 'text',
    do_objectID: models_1.generateID(`text:${name}-${textNodes.map((node) => node.content).join('')}`),
    frame,
    isFlippedHorizontal: false,
    isFlippedVertical: false,
    isLocked: false,
    isVisible: true,
    layerListExpandedType: sketch_file_format_ts_1.FileFormat1.LayerListExpanded.Undecided,
    name,
    nameIsFixed: false,
    resizingConstraint: resizeConstraint_1.makeResizeConstraint(resizingConstraint),
    resizingType: sketch_file_format_ts_1.FileFormat1.ResizeType.Stretch,
    rotation: 0,
    shouldBreakMaskChain: false,
    attributedString: makeAttributedString(bridge)(textNodes),
    style: exports.makeTextStyle(bridge)((textNodes[0] || { textStyles: {} }).textStyles, shadows),
    automaticallyDrawOnUnderlyingPath: false,
    dontSynchroniseWithSymbol: false,
    // NOTE(akp): I haven't fully figured out the meaning of glyphBounds
    glyphBounds: '',
    // glyphBounds: '{{0, 0}, {116, 17}}',
    lineSpacingBehaviour: sketch_file_format_ts_1.FileFormat1.LineSpacingBehaviour.ConsistentBaseline,
    textBehaviour: sketch_file_format_ts_1.FileFormat1.TextBehaviour.Fixed,
    booleanOperation: sketch_file_format_ts_1.FileFormat1.BooleanOperation.NA,
    exportOptions: {
        _class: 'exportOptions',
        exportFormats: [],
        includedLayerIds: [],
        layerOptions: 0,
        shouldTrim: false,
    },
    isFixedToViewport: false,
});
