var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
import * as TestRenderer from 'react-test-renderer';
import yoga from 'yoga-layout-prebuilt';
import { Context } from './utils/Context';
import { hasAnyDefined } from './utils/hasAnyDefined';
import { pick } from './utils/pick';
import { computeYogaTree } from './jsonUtils/computeYogaTree';
import { computeTextTree } from './jsonUtils/computeTextTree';
import { INHERITABLE_FONT_STYLES } from './utils/constants';
import { zIndex } from './utils/zIndex';
export var reactTreeToFlexTree = function (node, yogaNode, context) {
    var textNodes = [];
    var textStyle = context.getInheritedStyles();
    var newChildren = [];
    var style;
    var type;
    if (typeof node === 'string') {
        textNodes = computeTextTree(node, context);
        type = 'sketch_text';
    }
    else {
        style = node.props && node.props.style ? node.props.style : {};
        type = node.type || 'sketch_text';
        if (type === 'sketch_svg' && node.children) {
            // @ts-ignore
            newChildren = node.children;
        }
        else if (type === 'sketch_text') {
            // If current node is a Text node, add text styles to Context to pass down to
            // child nodes.
            if (node.props && node.props.style && hasAnyDefined(style, INHERITABLE_FONT_STYLES)) {
                var inheritableStyles = pick(style, INHERITABLE_FONT_STYLES);
                inheritableStyles.flexDirection = 'row';
                context.addInheritableStyles(inheritableStyles);
                textStyle = __assign(__assign({}, context.getInheritedStyles()), inheritableStyles);
            }
            // Compute Text Children
            textNodes = computeTextTree(node, context);
        }
        else if (node.children && node.children.length > 0) {
            // Recursion reverses the render stacking order
            // but that's actually fine because Sketch renders the first on top
            // Calculates zIndex order to match yoga
            var children = zIndex(node.children);
            for (var index = 0; index < children.length; index += 1) {
                var childComponent = children[index];
                var childNode = yogaNode.getChild(index);
                var renderedChildComponent = reactTreeToFlexTree(childComponent, childNode, context.forChildren());
                newChildren.push(renderedChildComponent);
            }
        }
    }
    return {
        type: type,
        style: style,
        textStyle: textStyle,
        layout: {
            left: yogaNode.getComputedLeft(),
            right: yogaNode.getComputedRight(),
            top: yogaNode.getComputedTop(),
            bottom: yogaNode.getComputedBottom(),
            width: yogaNode.getComputedWidth(),
            height: yogaNode.getComputedHeight(),
        },
        props: __assign(__assign({}, (typeof node !== 'string' ? node.props : {})), { textNodes: textNodes }),
        children: newChildren,
    };
};
export var buildTree = function (bridge) { return function (element) {
    var renderer;
    if (typeof TestRenderer.act !== 'undefined') {
        TestRenderer.act(function () {
            // synchronous callback
            renderer = TestRenderer.create(element);
        });
    }
    else {
        renderer = TestRenderer.create(element);
    }
    if (!renderer) {
        throw new Error('Cannot access react renderer');
    }
    var json = renderer.toJSON();
    if (!json) {
        throw new Error('Cannot render react element');
    }
    var yogaNode = computeYogaTree(bridge)(json, new Context());
    yogaNode.calculateLayout(undefined, undefined, yoga.DIRECTION_LTR);
    var tree = reactTreeToFlexTree(json, yogaNode, new Context());
    return tree;
}; };
