import { FileFormat1 as FileFormat } from '@sketch-hq/sketch-file-format-ts';
import { makeColorFromCSS, emptyGradient } from './models';
import { same } from '../utils/same';
import { makeVerticalBorder, makeHorizontalBorder } from './shapeLayers';
import { makeBorderOptions } from './style';
var DEFAULT_BORDER_COLOR = 'transparent';
export var DEFAULT_BORDER_STYLE = 'solid';
export var createUniformBorder = function (width, color, style, position, lineCapStyle, lineJoinStyle) {
    if (style === void 0) { style = DEFAULT_BORDER_STYLE; }
    if (position === void 0) { position = FileFormat.BorderPosition.Center; }
    if (lineCapStyle === void 0) { lineCapStyle = FileFormat.LineCapStyle.Butt; }
    if (lineJoinStyle === void 0) { lineJoinStyle = FileFormat.LineJoinStyle.Miter; }
    var borderOptions = makeBorderOptions(style, width, lineCapStyle, lineJoinStyle);
    var borders = [
        {
            _class: 'border',
            isEnabled: true,
            color: makeColorFromCSS(color),
            fillType: FileFormat.FillType.Color,
            position: position,
            thickness: width,
            contextSettings: {
                _class: 'graphicsContextSettings',
                blendMode: FileFormat.BlendMode.Normal,
                opacity: 1,
            },
            gradient: emptyGradient,
        },
    ];
    return { borderOptions: borderOptions, borders: borders };
};
export var createBorders = function (content, layout, style) {
    if (!style) {
        return [content];
    }
    var _a = style.borderTopWidth, borderTopWidth = _a === void 0 ? 0 : _a, _b = style.borderRightWidth, borderRightWidth = _b === void 0 ? 0 : _b, _c = style.borderBottomWidth, borderBottomWidth = _c === void 0 ? 0 : _c, _d = style.borderLeftWidth, borderLeftWidth = _d === void 0 ? 0 : _d, _e = style.borderTopColor, borderTopColor = _e === void 0 ? DEFAULT_BORDER_COLOR : _e, _f = style.borderRightColor, borderRightColor = _f === void 0 ? DEFAULT_BORDER_COLOR : _f, _g = style.borderBottomColor, borderBottomColor = _g === void 0 ? DEFAULT_BORDER_COLOR : _g, _h = style.borderLeftColor, borderLeftColor = _h === void 0 ? DEFAULT_BORDER_COLOR : _h, _j = style.borderTopStyle, borderTopStyle = _j === void 0 ? DEFAULT_BORDER_STYLE : _j, _k = style.borderRightStyle, borderRightStyle = _k === void 0 ? DEFAULT_BORDER_STYLE : _k, _l = style.borderBottomStyle, borderBottomStyle = _l === void 0 ? DEFAULT_BORDER_STYLE : _l, _m = style.borderLeftStyle, borderLeftStyle = _m === void 0 ? DEFAULT_BORDER_STYLE : _m;
    if (same(borderTopWidth, borderRightWidth, borderBottomWidth, borderLeftWidth) &&
        same(borderTopColor, borderRightColor, borderBottomColor, borderLeftColor) &&
        same(borderTopStyle, borderRightStyle, borderBottomStyle, borderLeftStyle)) {
        // all sides have same border width
        // in this case, we can do everything with just a single shape.
        if (borderTopStyle !== undefined && borderTopWidth !== null) {
            var borderOptions = makeBorderOptions(borderTopStyle, borderTopWidth);
            if (borderOptions && content.style) {
                content.style.borderOptions = borderOptions;
            }
        }
        if (borderTopWidth && borderTopWidth > 0 && content.style) {
            content.style.borders = createUniformBorder(borderTopWidth, borderTopColor, 'solid', FileFormat.BorderPosition.Outside).borders;
            var backingLayer = content.layers ? content.layers[0] : undefined;
            if (backingLayer) {
                backingLayer.frame.x += borderTopWidth;
                backingLayer.frame.y += borderTopWidth;
                backingLayer.frame.width -= borderTopWidth * 2;
                backingLayer.frame.height -= borderTopWidth * 2;
            }
        }
        return [content];
    }
    content.hasClippingMask = true;
    var layers = [content];
    if (borderTopWidth && borderTopWidth > 0) {
        var topBorder = makeHorizontalBorder(0, 0, layout.width, borderTopWidth, borderTopColor);
        topBorder.name = 'Border (top)';
        var borderOptions = makeBorderOptions(borderTopStyle, borderTopWidth);
        if (borderOptions && topBorder.style) {
            topBorder.style.borderOptions = borderOptions;
        }
        layers.push(topBorder);
    }
    if (borderRightWidth && borderRightWidth > 0) {
        var rightBorder = makeVerticalBorder(layout.width - borderRightWidth, 0, layout.height, borderRightWidth, borderRightColor);
        rightBorder.name = 'Border (right)';
        var borderOptions = makeBorderOptions(borderRightStyle, borderRightWidth);
        if (borderOptions && rightBorder.style) {
            rightBorder.style.borderOptions = borderOptions;
        }
        layers.push(rightBorder);
    }
    if (borderBottomWidth && borderBottomWidth > 0) {
        var bottomBorder = makeHorizontalBorder(0, layout.height - borderBottomWidth, layout.width, borderBottomWidth, borderBottomColor);
        bottomBorder.name = 'Border (bottom)';
        var borderOptions = makeBorderOptions(borderBottomStyle, borderBottomWidth);
        if (borderOptions && bottomBorder.style) {
            bottomBorder.style.borderOptions = borderOptions;
        }
        layers.push(bottomBorder);
    }
    if (borderLeftWidth && borderLeftWidth > 0) {
        var leftBorder = makeVerticalBorder(0, 0, layout.height, borderLeftWidth, borderLeftColor);
        leftBorder.name = 'Border (left)';
        var borderOptions = makeBorderOptions(borderLeftStyle, borderLeftWidth);
        if (borderOptions && leftBorder.style) {
            leftBorder.style.borderOptions = borderOptions;
        }
        layers.push(leftBorder);
    }
    return layers;
};
