"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ImageRenderer = void 0;
const SketchRenderer_1 = require("./SketchRenderer");
const getImageDataFromURL_1 = require("../utils/getImageDataFromURL");
// import processTransform from './processTransform';
const models_1 = require("../jsonUtils/models");
const shapeLayers_1 = require("../jsonUtils/shapeLayers");
const borders_1 = require("../jsonUtils/borders");
function extractURLFromSource(source) {
    if (typeof source === 'string') {
        return source;
    }
    return (source || {}).uri;
}
class ImageRenderer extends SketchRenderer_1.SketchRenderer {
    renderBackingLayers({ layout, style, props, }) {
        let layers = [];
        const { borderTopLeftRadius = 0, borderTopRightRadius = 0, borderBottomRightRadius = 0, borderBottomLeftRadius = 0, } = style;
        const url = extractURLFromSource(props.source);
        const image = getImageDataFromURL_1.getImageDataFromURL(this.platformBridge)(url);
        const fillImage = models_1.makeJSONDataReference(image);
        const frame = models_1.makeRect(0, 0, layout.width, layout.height);
        const radii = [
            borderTopLeftRadius,
            borderTopRightRadius,
            borderBottomRightRadius,
            borderBottomLeftRadius,
        ];
        const shapeLayer = shapeLayers_1.makeRectShapeLayer(0, 0, layout.width, layout.height, radii);
        const fills = [models_1.makeImageFill(fillImage, props.resizeMode)];
        const content = shapeLayers_1.makeShapeGroup(frame, [shapeLayer], style, props.shadows, fills);
        // try to keep a constant ID based on the URL
        content.do_objectID = models_1.generateID(url);
        const contents = borders_1.createBorders(content, layout, style);
        layers = layers.concat(contents);
        return layers;
    }
}
exports.ImageRenderer = ImageRenderer;
