"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SvgRenderer = void 0;
const ViewRenderer_1 = require("./ViewRenderer");
const makeSvgLayer_1 = require("../jsonUtils/makeSvgLayer");
const snakeExceptions = [
    'gradientUnits',
    'gradientTransform',
    'patternUnits',
    'patternTransform',
    'stdDeviation',
    'numOctaves',
    'specularExponent',
    'specularConstant',
    'surfaceScale',
    'viewBox',
];
function toSnakeCase(string) {
    if (string === 'href') {
        return 'xlink:href';
    }
    if (snakeExceptions.indexOf(string) !== -1) {
        return string;
    }
    return string.replace(/([A-Z])/g, ($1) => `-${$1.toLowerCase()}`);
}
function makeSvgString(el) {
    if (typeof el === 'string') {
        return el;
    }
    const { type, props, children } = el;
    if (props && props.textNodes && props.textNodes.length) {
        return props.textNodes.reduce((prev, textNode) => prev + textNode.content, '');
    }
    if (!type || type.indexOf('svg_') !== 0) {
        throw new Error(`Could not render type '${type}'. Make sure to only have <Svg.*> components inside <Svg>.`);
    }
    const cleanedType = type.slice(4);
    const attributes = Object.keys(props || {}).reduce(
    // @ts-ignore
    (prev, k) => (props[k] ? `${prev} ${toSnakeCase(k)}="${props[k]}"` : prev), '');
    let string = `<${cleanedType}${attributes}`;
    if (!children || !children.length) {
        string += '/>\n';
    }
    else {
        string += '>\n';
        string += (children || []).reduce((prev, c) => `${prev}  ${makeSvgString(c)}`, '');
        string += `</${cleanedType}>\n`;
    }
    return string;
}
class SvgRenderer extends ViewRenderer_1.ViewRenderer {
    getDefaultGroupName(props) {
        return props.name || 'Svg';
    }
    renderBackingLayers(node) {
        const layers = super.renderBackingLayers(node);
        const { layout, props, children, style } = node;
        // add the "xmlns:xlink" namespace so we can use `href`
        props['xmlns:xlink'] = 'http://www.w3.org/1999/xlink';
        const svgString = makeSvgString({
            type: 'svg_svg',
            props,
            children,
            style,
            layout,
        });
        const svgLayer = makeSvgLayer_1.makeSvgLayer(layout, 'Shape', svgString);
        layers.push(svgLayer);
        return layers;
    }
}
exports.SvgRenderer = SvgRenderer;
