"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ViewRenderer = void 0;
const SketchRenderer_1 = require("./SketchRenderer");
const models_1 = require("../jsonUtils/models");
const shapeLayers_1 = require("../jsonUtils/shapeLayers");
const borders_1 = require("../jsonUtils/borders");
const hasAnyDefined_1 = require("../utils/hasAnyDefined");
const VISIBLE_STYLES = [
    'shadowColor',
    'shadowOffset',
    'shadowOpacity',
    'shadowRadius',
    'shadowSpread',
    'backgroundColor',
    'borderColor',
    'borderTopColor',
    'borderRightColor',
    'borderBottomColor',
    'borderLeftColor',
    'borderStyle',
    'borderTopStyle',
    'borderRightStyle',
    'borderBottomStyle',
    'borderLeftStyle',
    'borderWidth',
    'borderTopWidth',
    'borderRightWidth',
    'borderBottomWidth',
    'borderLeftWidth',
];
const OVERFLOW_STYLES = ['overflow', 'overflowX', 'overflowY'];
class ViewRenderer extends SketchRenderer_1.SketchRenderer {
    getDefaultGroupName(_props) {
        return 'View';
    }
    renderBackingLayers({ layout, style, props, }) {
        let layers = [];
        // NOTE(lmr): the group handles the position, so we just care about width/height here
        const { borderTopLeftRadius = 0, borderTopRightRadius = 0, borderBottomRightRadius = 0, borderBottomLeftRadius = 0, } = style;
        if (!hasAnyDefined_1.hasAnyDefined(style, VISIBLE_STYLES)) {
            return layers;
        }
        const frame = models_1.makeRect(0, 0, layout.width, layout.height);
        const radii = [
            borderTopLeftRadius,
            borderTopRightRadius,
            borderBottomRightRadius,
            borderBottomLeftRadius,
        ];
        const shapeLayer = shapeLayers_1.makeRectShapeLayer(0, 0, layout.width, layout.height, radii, props.resizingConstraint);
        const content = shapeLayers_1.makeShapeGroup(frame, [shapeLayer], style, props.shadows);
        if (hasAnyDefined_1.hasAnyDefined(style, OVERFLOW_STYLES)) {
            if (style.overflow === 'hidden' ||
                style.overflow === 'scroll' ||
                style.overflowX === 'hidden' ||
                style.overflowX === 'scroll' ||
                style.overflowY === 'hidden' ||
                style.overflowY === 'scroll') {
                content.hasClippingMask = true;
            }
        }
        const contents = borders_1.createBorders(content, layout, style);
        layers = layers.concat(contents);
        return layers;
    }
}
exports.ViewRenderer = ViewRenderer;
