"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.TextStyles = void 0;
const getSketchVersion_1 = require("../utils/getSketchVersion");
const hashStyle_1 = require("../utils/hashStyle");
const getDocument_1 = require("../utils/getDocument");
const sharedTextStyles_1 = require("../utils/sharedTextStyles");
const textLayers_1 = require("../jsonUtils/textLayers");
const pick_1 = require("../utils/pick");
const constants_1 = require("../utils/constants");
let _styles = {};
const _byName = {};
exports.TextStyles = (getDefaultBridge) => ({
    registerStyle(name, style, platformBridge = getDefaultBridge()) {
        const safeStyle = pick_1.pick(style, constants_1.INHERITABLE_FONT_STYLES);
        const hash = hashStyle_1.hashStyle(safeStyle);
        const sketchStyle = textLayers_1.makeTextStyle(platformBridge)(safeStyle);
        const sharedObjectID = sharedTextStyles_1.sharedTextStyles.addStyle(name, sketchStyle);
        // FIXME(gold): side effect :'(
        _byName[name] = hash;
        _styles[hash] = {
            cssStyle: safeStyle,
            name,
            sketchStyle,
            sharedObjectID,
        };
    },
    create(styles, options = {}, platformBridge = getDefaultBridge()) {
        const { clearExistingStyles, document } = options;
        const doc = getDocument_1.getDocument(document);
        const sketchVersion = getSketchVersion_1.getSketchVersion();
        if (sketchVersion !== 'NodeJS' && sketchVersion < 50) {
            if (doc) {
                doc.showMessage('💎 Requires Sketch 50+ 💎');
            }
            return {};
        }
        sharedTextStyles_1.sharedTextStyles.setDocument(doc);
        if (clearExistingStyles) {
            _styles = {};
            sharedTextStyles_1.sharedTextStyles.setStyles([]);
        }
        Object.keys(styles).forEach((name) => this.registerStyle(name, styles[name], platformBridge));
        return _styles;
    },
    resolve(style) {
        if (!style) {
            return undefined;
        }
        const safeStyle = pick_1.pick(style, constants_1.INHERITABLE_FONT_STYLES);
        const hash = hashStyle_1.hashStyle(safeStyle);
        return _styles[hash];
    },
    get(name, document) {
        const hash = _byName[name];
        const style = _styles[hash];
        if (style) {
            return style.cssStyle;
        }
        return sharedTextStyles_1.sharedTextStyles.getStyle(name, document ? getDocument_1.getDocument(document) : undefined);
    },
    clear() {
        _styles = {};
        sharedTextStyles_1.sharedTextStyles.setStyles([]);
    },
    toJSON() {
        return Object.keys(_styles).map((k) => ({
            _class: 'sharedStyle',
            do_objectID: _styles[k].sharedObjectID,
            name: _styles[k].name,
            value: _styles[k].sketchStyle,
        }));
    },
    styles() {
        return _styles;
    },
});
