"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getSymbolComponentByName = exports.getSymbolComponentById = exports.getSymbolMasterById = exports.getSymbolMasterByName = exports.makeSymbol = exports.createSymbolInstanceClass = exports.injectSymbols = void 0;
const React = __importStar(require("react"));
const PropTypes = __importStar(require("prop-types"));
const fromSJSON_1 = require("./jsonUtils/sketchJson/fromSJSON");
const toSJSON_1 = require("./jsonUtils/sketchJson/toSJSON");
const stylesheet_1 = require("./stylesheet");
const models_1 = require("./jsonUtils/models");
const ViewStylePropTypes_1 = require("./components/ViewStylePropTypes");
const ResizingConstraintPropTypes_1 = require("./components/ResizingConstraintPropTypes");
const buildTree_1 = require("./buildTree");
const flexToSketchJSON_1 = require("./flexToSketchJSON");
const render_1 = require("./render");
const resets_1 = require("./resets");
const getDocument_1 = require("./utils/getDocument");
const getSketchVersion_1 = require("./utils/getSketchVersion");
let id = 0;
const nextId = () => ++id;
const displayName = (Component) => Component.displayName || Component.name || `UnknownSymbol${nextId()}`;
let hasInitialized = false;
const symbolsRegistry = {};
let existingSymbols = [];
const layers = {};
function msListToArray(pageList) {
    const out = [];
    for (let i = 0; i < pageList.length; i++) {
        out.push(pageList[i]);
    }
    return out;
}
const getSymbolsPage = (documentData) => documentData.symbolsPageOrCreateIfNecessary();
function exists(x) {
    return !!x;
}
const getExistingSymbols = (documentData) => {
    if (!hasInitialized) {
        hasInitialized = true;
        const symbolsPage = getSymbolsPage(documentData);
        existingSymbols = msListToArray(symbolsPage.layers())
            .map((x) => {
            const symbolJson = toSJSON_1.toSJSON(x);
            if (!symbolJson || symbolJson._class !== 'symbolMaster') {
                return undefined;
            }
            layers[symbolJson.symbolID] = x;
            return symbolJson;
        })
            .filter(exists);
        existingSymbols.forEach((symbolMaster) => {
            if (symbolMaster._class !== 'symbolMaster')
                return;
            if (symbolMaster.name in symbolsRegistry)
                return;
            symbolsRegistry[symbolMaster.name] = symbolMaster;
        });
    }
    return existingSymbols;
};
exports.injectSymbols = (document) => {
    if (getSketchVersion_1.getSketchVersion() === 'NodeJS') {
        console.error('Cannot inject symbols in NodeJS');
        return;
    }
    // if hasInitialized is false then makeSymbol has not yet been called
    // so we don't have anything to inject
    if (hasInitialized) {
        const documentData = getDocument_1.getDocumentData(document);
        if (!documentData) {
            return;
        }
        const currentPage = documentData.currentPage();
        const symbolsPage = getSymbolsPage(documentData);
        let left = 0;
        Object.keys(symbolsRegistry).forEach((key) => {
            const symbolMaster = symbolsRegistry[key];
            symbolMaster.frame.y = 0;
            symbolMaster.frame.x = left;
            left += symbolMaster.frame.width + 20;
            const newLayer = fromSJSON_1.fromSJSON(symbolMaster, '119');
            layers[symbolMaster.symbolID] = newLayer;
        });
        // Clear out page layers to prepare for re-render
        resets_1.resetLayer(symbolsPage);
        render_1.renderLayers(Object.keys(layers).map((k) => layers[k]), symbolsPage);
        documentData.setCurrentPage(currentPage);
    }
};
const SymbolInstancePropTypes = {
    style: PropTypes.shape(ViewStylePropTypes_1.ViewStylePropTypes),
    name: PropTypes.string,
    overrides: PropTypes.objectOf(PropTypes.oneOfType([PropTypes.string, PropTypes.func])),
    resizingConstraint: PropTypes.shape(Object.assign({}, ResizingConstraintPropTypes_1.ResizingConstraintPropTypes)),
};
exports.createSymbolInstanceClass = (symbolMaster) => {
    var _a;
    return _a = class SymbolInstance extends React.Component {
            render() {
                return (React.createElement("sketch_symbolinstance", { symbolID: symbolMaster.symbolID, name: this.props.name || symbolMaster.name, style: stylesheet_1.StyleSheet.flatten(this.props.style), resizingConstraint: this.props.resizingConstraint, overrides: this.props.overrides }));
            }
        },
        _a.displayName = `SymbolInstance(${symbolMaster.name})`,
        _a.propTypes = SymbolInstancePropTypes,
        _a.symbolID = symbolMaster.symbolID,
        _a.masterName = symbolMaster.name,
        _a;
};
const SymbolMasterPropTypes = {
    style: PropTypes.shape(ViewStylePropTypes_1.ViewStylePropTypes),
    name: PropTypes.string,
};
exports.makeSymbol = (bridge) => (Component, symbolProps, document) => {
    if (!hasInitialized && getSketchVersion_1.getSketchVersion() !== 'NodeJS') {
        const documentData = getDocument_1.getDocumentData(document);
        if (documentData) {
            getExistingSymbols(documentData);
        }
    }
    const masterName = (typeof symbolProps === 'string' ? symbolProps : (symbolProps || {}).name) ||
        displayName(Component);
    const existingSymbol = existingSymbols.find((symbolMaster) => symbolMaster.name === masterName);
    const symbolID = existingSymbol
        ? existingSymbol.symbolID
        : models_1.generateID(`symbolID:${masterName}`, !!masterName);
    const symbolMaster = flexToSketchJSON_1.flexToSketchJSON(bridge)(buildTree_1.buildTree(bridge)(React.createElement("sketch_symbolmaster", Object.assign({}, (typeof symbolProps !== 'string' ? symbolProps || {} : {}), { symbolID: symbolID, name: masterName }),
        React.createElement(Component, null))));
    symbolsRegistry[symbolID] = symbolMaster;
    return exports.createSymbolInstanceClass(symbolMaster);
};
function tryGettingSymbolMasterInDocumentByName(name, document) {
    const documentData = getDocument_1.getDocumentData(document);
    if (!documentData) {
        return undefined;
    }
    const symbols = documentData.symbolMap();
    const symbol = Object.keys(symbols).find((k) => symbols[k].name() === name);
    if (!symbol) {
        return undefined;
    }
    return toSJSON_1.toSJSON(symbol);
}
function tryGettingSymbolMasterInDocumentById(symbolID, document) {
    const documentData = getDocument_1.getDocumentData(document);
    if (!documentData) {
        return undefined;
    }
    const symbol = documentData.symbolMap()[symbolID];
    if (!symbol) {
        return undefined;
    }
    return toSJSON_1.toSJSON(symbol);
}
exports.getSymbolMasterByName = (name, document) => {
    const symbolID = name
        ? Object.keys(symbolsRegistry).find((key) => String(symbolsRegistry[key].name) === name)
        : '';
    if (typeof symbolID === 'undefined' && name && getSketchVersion_1.getSketchVersion() !== 'NodeJS') {
        return tryGettingSymbolMasterInDocumentByName(name, document);
    }
    if (typeof symbolID === 'undefined') {
        throw new Error('##FIXME## NO MASTER FOR THIS SYMBOL NAME');
    }
    return symbolsRegistry[symbolID];
};
exports.getSymbolMasterById = (symbolID, document) => {
    let symbolMaster = symbolID ? symbolsRegistry[symbolID] : undefined;
    if (typeof symbolMaster === 'undefined' && symbolID && getSketchVersion_1.getSketchVersion() !== 'NodeJS') {
        symbolMaster = tryGettingSymbolMasterInDocumentById(symbolID, document);
    }
    if (typeof symbolMaster === 'undefined') {
        throw new Error('##FIXME## NO MASTER WITH THAT SYMBOL ID');
    }
    return symbolMaster;
};
exports.getSymbolComponentById = (symbolID, document) => {
    const symbolMaster = exports.getSymbolMasterById(symbolID, document);
    if (!symbolMaster) {
        return undefined;
    }
    return exports.createSymbolInstanceClass(symbolMaster);
};
exports.getSymbolComponentByName = (masterName, document) => {
    const symbolMaster = exports.getSymbolMasterByName(masterName, document);
    if (!symbolMaster) {
        return undefined;
    }
    return exports.createSymbolInstanceClass(symbolMaster);
};
