"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.sharedTextStyles = void 0;
const invariant_1 = __importDefault(require("invariant"));
const fromSJSON_1 = require("../../jsonUtils/sketchJson/fromSJSON");
const toSJSON_1 = require("../../jsonUtils/sketchJson/toSJSON");
const models_1 = require("../../jsonUtils/models");
const textLayers_1 = require("../../jsonUtils/textLayers");
class TextStyles {
    constructor() {
        this._document = null;
    }
    setDocument(doc) {
        invariant_1.default(doc, 'Please provide a sketch document reference');
        this._document = doc;
        return this;
    }
    setStyles(styles) {
        invariant_1.default(this._document, 'Please provide a sketch document reference');
        this._document.documentData().layerTextStyles().setObjects(styles);
        return this;
    }
    addStyle(name, style) {
        const { _document } = this;
        invariant_1.default(_document, 'Please provide a sketch document reference');
        const nativeStyle = fromSJSON_1.fromSJSON(style, '119');
        const container = _document.documentData().layerTextStyles();
        let sharedStyle;
        // Sketch < 50
        if (container.addSharedStyleWithName_firstInstance) {
            sharedStyle = container.addSharedStyleWithName_firstInstance(name, nativeStyle);
        }
        else {
            const allocator = MSSharedStyle.alloc();
            // Sketch 50, 51
            if (allocator.initWithName_firstInstance) {
                sharedStyle = allocator.initWithName_firstInstance(name, nativeStyle);
            }
            else {
                sharedStyle = allocator.initWithName_style(name, nativeStyle);
            }
            container.addSharedObject(sharedStyle);
        }
        sharedStyle.objectID = models_1.generateID(`sharedStyle:${name}`, !!name);
        // NOTE(gold): the returned object ID changes after being added to the store
        // _don't_ rely on the object ID we pass to it, but we have to have one set
        // otherwise Sketch crashes
        return String(sharedStyle.objectID());
    }
    getStyle(name, document) {
        const { _document } = this;
        const doc = document || _document;
        invariant_1.default(doc, 'Please provide a sketch document reference');
        const sharedStyles = doc.documentData().layerTextStyles().objects();
        let foundStyle = undefined;
        for (let i = 0; i < sharedStyles.length; i++) {
            if (String(sharedStyles[i].name()) === String(name)) {
                foundStyle = sharedStyles[i].style();
            }
        }
        if (!foundStyle) {
            return undefined;
        }
        const style = toSJSON_1.toSJSON(foundStyle);
        return textLayers_1.parseTextStyle(style);
    }
}
exports.sharedTextStyles = new TextStyles();
